/****************************************************************************
*
*     Firefly X10 Linux Kernel driver
*     File:    x10_driver.c
*     Version: 1.4
*     Copyright 2005 Heber Limited (http://www.heber.co.uk)
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2, or (at your option) any
*     later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*     General Public License for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     675 Mass Ave, Cambridge, MA 02139, USA.
*
*     Changes:
*     1.4 MJB       Previously, when USB communications between the PC and
*                   the X10 were heavy (e.g. constant reading/writing SRAM),
*                   and the X10 was suddenly disconnected, a system crash
*                   could occur. This would only happen under the Linux
*                   2.4.x kernel. This bug has been fixed by adding
*                   semaphores around any USB communication functions
*                   and ensuring that the semaphores are free prior to
*                   freeing resources in X10_usb_disconnect(), 4/3/2005
*     1.3 MJB       - Uses 8051 code version 41. Fixed various cctalk bugs
*                   and X10 fault whereby it hanged when a get descriptor
*                   call was made
*                   - Added new debug information to this driver, 9/7/2004
*     1.2 MJB       - Uses 8051 code version 31. This fixes a bug whereby
*                   the X10 would hang during initialisation if the SRAM
*                   had been cleared.
*                   - Renamed file from "driver_init.c" to "x10_driver.c".
*                   This was to prevent any confusion when compiling into
*                   a static kernel.
*                   - Changed function names init_module() and
*                   cleanup_module() to X10_init_module() and
*                   X10_cleanup_module() to allow module to build into
*                   static kernel properly, 11/11/2003
*     1.1 MJB       Released under GPL, 29/9/2003
*     1.0 MJB       First version, 30/7/2003
*
****************************************************************************/


#include <asm/semaphore.h>

#include <linux/module.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/interrupt.h>
#include <linux/usb.h>
#include <linux/proc_fs.h>
//#define _X10_USB_DEBUG
#include <asm/uaccess.h> 		//2.6 change

#include "17207.h"					// Contains driver title and issue no.
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
	#define	OLD_KERNEL
#else

#include <linux/moduleparam.h>

#endif


//#define __NO_VERSION__


#define DRIVER_DESC                 "Firefly X10 Kernel Driver"
#define DRIVER_AUTHOR               "Heber Limited <http://www.heber.co.uk>"
#define DRIVER_SUPPORTED_DEVICES    "X10"
#define DRIVER_VERSION "v0.1"
// The X10 Kernel driver is under a GPL release.
MODULE_LICENSE("GPL");

// Specify some module information
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_SUPPORTED_DEVICE(DRIVER_SUPPORTED_DEVICES);

#define HEBER			(0x0fb6)
#define X10				(0x3fc5)
#define X10_LOADER		(0x3fc6)
#define LOADER			0
#define IOCARD			1
static int X10_major_number = 0;

#define BUFFER_LENGTH 128
#define UNDER_RUN     1
#define OVER_RUN      2
#ifndef FALSE
#define FALSE (0)
#endif

// Response timeout * HZ
#define TIMEOUT_MULT_HZ		100.5
#define MAX_USB_RETRIES			10
// X10 pipe definitions
//#define USB_PIPE_SRAM					2
#define USB_PIPE_SRAM					6
#define NUMBER_OF_PIPES					7
#define BACK_CHANNEL_PIPE		6

// SRAM 8051 constants
#define USB_MSG_WRITE_SRAM								71
#define USB_MESSAGE_CODE_DOWNLOAD_START					72
#define USB_MESSAGE_CODE_DOWNLOAD_COMPLETED				73
#define USB_MESSAGE_IS_CODE_DOWNLOAD_COMPLETE			74

#define USB_MESSAGE_EXECUTION_SUCCESS	0
#define ANCHOR_LOAD_INTERNAL			0xA0

// This command is not implemented in the core.  Requires firmware
#define ANCHOR_LOAD_EXTERNAL			0xA3

// This is the highest internal RAM address for the AN2131Q
#define MAX_INTERNAL_ADDRESS			0x1B3F

//#define INTERNAL_RAM( address ) ( ( address <= MAX_INTERNAL_ADDRESS ) ? 1 : 0 )
inline int INTERNAL_RAM( unsigned short int address )
{
	return address <= MAX_INTERNAL_ADDRESS ;
}

// EZ-USB Control and Status Register.  Bit 0 controls 8051 reset
#define CPUCS_REG						0x7F92

#include "x10locode.c"		/* I make these C files as they generate code */
#include "x10hicode.c"		/* The Hi and lo files are autogenerated from an intel hex file with the 8051 code in it */
#include "loader.c"			/* do as an include rather than seperate compilation to keep symbols static*/
							/* This is to avoid poluting the kernel namespace on loading	*/

static const unsigned long  ProfileRanges[]= {50,100,150,200,250,300,350,400,450,500,1000,2000,3000,4000} ;
#define NUMBER_OF_PROFILE_RANGES  (sizeof(ProfileRanges)/sizeof(ProfileRanges[0]))

struct X10ProfileEntry
{
	int opCode ;
	unsigned long Counts[NUMBER_OF_PROFILE_RANGES];
	struct X10ProfileEntry * Next  ;
} ;

struct Pipe
{
	wait_queue_head_t queue;
	int InUse;
	struct timespec StartTime ;
	struct X10ProfileEntry * profile_data ;
};
static int LogLevel;

/*====================================================================
Structures to contain information about previous X10 transaction timings

====================================================================*/
#ifdef _X10_USB_DEBUG
#define LOG_BUFFER_SIZE				64
#define NUM_TRANSACTIONS			50
static struct X10ProfileEntry * make_profile_entry(int opCode)
{
	struct X10ProfileEntry * Ptr ;

	Ptr = kmalloc(sizeof(struct X10ProfileEntry),GFP_KERNEL);
	if( Ptr )
	{
		Ptr->opCode = opCode ;
		memset(Ptr->Counts,0,sizeof(Ptr->Counts));
		Ptr->Next=NULL;
	}

	return Ptr ;
}

/*====================================================================
====================================================================*/
static struct X10ProfileEntry * get_profile_entry( int opCode, struct X10ProfileEntry * * Base )
{
	struct X10ProfileEntry * Ptr = *Base ;
	while (Ptr)
	{
		if (Ptr->opCode==opCode)
			return Ptr ;
		Ptr=Ptr->Next ;
	}
	Ptr = make_profile_entry(opCode);
	Ptr->Next = *Base ;
	*Base = Ptr ;

	return Ptr ;
}

/*====================================================================
====================================================================*/
static void add_profile_entry(int opCode, struct Pipe * pipe,unsigned long elapsed)
{
	unsigned int x;

	struct X10ProfileEntry * ptr = get_profile_entry(opCode, &pipe->profile_data);
	if( ptr )
	{
		for ( x=0; x<NUMBER_OF_PROFILE_RANGES; x++ )
		{
			if ( elapsed < ProfileRanges[x] )
			{
					ptr->Counts[x]++;
					return;
			}
		}
		ptr->Counts[NUMBER_OF_PROFILE_RANGES-1]++;
	}
}

/*====================================================================
====================================================================*/
static void delete_profile_entries( struct X10ProfileEntry ** Base )
{
	struct X10ProfileEntry * Ptr = *Base;

	while( Ptr )
	{
		Base = &Ptr->Next ;
		kfree( Ptr );
		Ptr = *Base ;
	}
}


/*====================================================================
====================================================================*/
typedef struct _LogEntry
{
	char TxBuffer[LOG_BUFFER_SIZE];
	char RxBuffer[LOG_BUFFER_SIZE];
	int TxLength;
	int RxLength;
	unsigned long TxTime;
	unsigned long RxTime;
	int PipeNum;
} LogEntry;

/*====================================================================
====================================================================*/
typedef struct _TransHistroy
{
	LogEntry LogEntries[NUM_TRANSACTIONS];
	volatile	int CurrentLog;
	volatile	int UsedLogs;
	struct semaphore LogSemaphore;
} TransHistory;

static TransHistory TxHistory;

/*====================================================================
====================================================================*/
static void DisplayLogs( void )
{
	int i, usedLogs, currentLog;
	static int dumped= 0;
	if (dumped)
		return ;
	dumped = 1 ;
	down( &TxHistory.LogSemaphore );

	usedLogs = TxHistory.UsedLogs;
	currentLog = TxHistory.CurrentLog;
	while ( usedLogs )
	{
		usedLogs --;

		if ( currentLog ) currentLog --;
		else currentLog = NUM_TRANSACTIONS - 1;
	}

	usedLogs = TxHistory.UsedLogs;
	while ( usedLogs )
	{
		printk( "<1>Tx time = %ld, Buffer -%d = [ ", TxHistory.LogEntries[currentLog].TxTime, usedLogs );
		for ( i = 0; i < TxHistory.LogEntries[currentLog].TxLength; i ++ )
		{
			printk( "%02X ", TxHistory.LogEntries[currentLog].TxBuffer[i] );
		}
		printk( "]\n" );

		printk( "<1>Rx time = %ld, Buffer -%d = [ ", TxHistory.LogEntries[currentLog].RxTime, usedLogs );
		for ( i = 0; i < TxHistory.LogEntries[currentLog].RxLength; i ++ )
		{
			printk( "%02X ", TxHistory.LogEntries[currentLog].RxBuffer[i] );
		}
		printk( "]\n" );

		usedLogs --;

		currentLog ++;
		if ( currentLog == NUM_TRANSACTIONS ) currentLog = 0;
	}

	TxHistory.UsedLogs = 0;
	TxHistory.CurrentLog = 0;

	up( &TxHistory.LogSemaphore );
}
#endif

/*====================================================================
Defines a structure to contain all the data we need about an individual X10
====================================================================*/
struct driver_context
{
	int Type							;
	int Number							;
	int DownloadFinished				;
	struct Pipe Pipes[NUMBER_OF_PIPES]	;
	struct usb_device * dev				;
	char True							;		//Create this structure in DMA accessible memory
	char False							;		//so we predefine flags for the Reset Command
	//devfs_handle_t DevFileSystemHandle;
	struct driver_context * next		;
	struct timeval LastInputPacketTime ;		//Rememeber the time of the last IO packet
	char   LastIOPacket[64]				;
	size_t LastIOPacketSize				;
	pid_t kx10d_pid						;
	struct semaphore	InputBufferSemaphore;
	struct semaphore    USBCommunicationSemaphore;
	struct completion		kx10d_exited;
};

/*====================================================================
The driver has a linked list of X10's
=====================================================================*/
static struct driver_context * ContextList = NULL;
static int DoUserX10IO( struct driver_context * context, X10_COMMAND *UserCommand );

/*====================================================================
Llseek reset the offset
=====================================================================*/
static loff_t X10_llseek( struct file * file_ptr, loff_t offset, int mode )
{
	switch (mode)
	{
	case 0:
		file_ptr->f_pos= offset ;
		return file_ptr->f_pos ;
	case 1:
		file_ptr->f_pos+= offset ;
		return file_ptr->f_pos ;
	default :
		return -EINVAL ;
	}

}

/*====================================================================
Allow a user application to read the return thread results
=====================================================================*/
static ssize_t X10_read( struct file * file_ptr, char * user_data_ptr, size_t user_data_size, loff_t *position_ptr )
{
	struct driver_context * context = file_ptr->private_data;
	size_t transfer_size, return_value;

	loff_t p = *position_ptr,second_offset ;
	up(&context->InputBufferSemaphore);		//ToDo make interruptable???
	transfer_size = context->LastIOPacketSize+sizeof(struct timeval)-p;
	transfer_size = (user_data_size<transfer_size)?user_data_size:transfer_size;
	const size_t first_block_size = sizeof(struct timeval) ;

	if ( context->LastIOPacketSize == 0 )
	{
		return_value = 0 ;
#ifdef _X10_USB_DEBUG
		printk( "No X10 data available yet during read\n" );
#endif
		goto end ;
	}
	return_value=transfer_size;
	if ( p<first_block_size )
	{
		if( copy_to_user( user_data_ptr, ((unsigned char *)&context->LastInputPacketTime)+p,first_block_size-p  ) )
		{
#ifdef _X10_USB_DEBUG
			printk( "copy_to_user failed during read\n" );
#endif
			return_value=( -EFAULT );
			goto end;
		}
	}
	second_offset = 0;
	if ( p>first_block_size )
	{
		second_offset = p-first_block_size ;
	}
	if( copy_to_user( user_data_ptr+sizeof(struct timeval), ((unsigned char *) &context->LastIOPacket)+second_offset , context->LastIOPacketSize-second_offset ) )
	{
#ifdef _X10_USB_DEBUG
		printk( "copy_to_user failed during read\n" );
#endif
		return_value=( -EFAULT );
	}
end:
	down( &context->InputBufferSemaphore );
	*position_ptr+=transfer_size ;

	return return_value;
}

/*====================================================================
All IO is performed using IOCTL so write is just a stub
=====================================================================*/
static ssize_t X10_write( struct file * file_ptr, const char * user_data_ptr, size_t user_data_size, loff_t * position_ptr )
{
#ifdef _X10_USB_DEBUG
	printk( "<1> X10 called X10 driver write\n" );
#endif

	return 0;
}

/*====================================================================
All the action starts here
=====================================================================*/
static int X10_ioctl( struct inode * inode_ptr, struct file * file_ptr, unsigned int command, unsigned long arg )
{
	struct driver_context * context = file_ptr->private_data;
	int ReturnValue;
	X10_COMMAND * X10Command = (X10_COMMAND *) arg;

	// Check that we know the IOCTL code and the user buffer is accessible
	if( ( command == IOCTL_X10_COMMAND ) && access_ok( VERIFY_WRITE, X10Command, sizeof( X10_COMMAND ) ) )
	{
		ReturnValue = DoUserX10IO( context, X10Command );
	}
	else
	{
#ifdef _X10_USB_DEBUG
		printk("X10 Failed to access user buffer \n");
#endif
		ReturnValue = -ENOTTY;
	}

	LogLevel = 0;

	return( ReturnValue );
}

/*====================================================================
X10 open the driver supports multiple X10's so we must track down the
right one. Get the device minor number and then search the linked list
of driver contexts for a device that matches. If we find a match
then we record the context in the struct file's private data member and
allow the open to succeed
=====================================================================*/
static int X10_open( struct inode * inode_ptr, struct file * file_ptr )
{
	int MinorNumber = MINOR( inode_ptr->i_rdev );
	struct driver_context * ptr = ContextList;

#ifdef _X10_USB_DEBUG
	printk( "<1> Open called X10 driver Minor Number =%d\n",MinorNumber );
#endif

	while( ptr && ptr->Number != MinorNumber )
	{
		ptr = ptr->next;
	}

	if( ptr && ptr->DownloadFinished)
	{
		file_ptr->private_data = ptr;
#ifdef _X10_USB_DEBUG
		printk( "Open succeeded MinorNumber =%d \n", ptr->Number );
#endif
		#ifdef OLD_KERNEL
		MOD_INC_USE_COUNT;	//	Not needed for 2.6 kernels
		#endif
		return( 0 );
	}
	else
	{
		return( -ENODEV );	//Failed to find a connected X10 with the right device number
	}
}

/*====================================================================
The close just decrements the driver modules reference count
=====================================================================*/
static int X10_release( struct inode * inode_ptr, struct file * file_ptr )
{
#ifdef _X10_USB_DEBUG
	printk( "<1> Release Called X10 driver\n" );
#endif
	#ifdef OLD_KERNEL
	MOD_DEC_USE_COUNT; 	//not needed for 2.6
	#endif
	return( 0 );
}


/*====================================================================
Defines a table of file operations for an X10 device
This is a table of callbacks
=====================================================================*/
static struct file_operations X10_file_operations =
{
                                          /*struct module * owner ; */
    llseek:          X10_llseek,         /*loff_t(*llseek)(struct file *,loff_t,int);*/
    read:            X10_read,           /*ssize_t (*read)(struct file *,char *,size_t,loff_t *);*/
    write:           X10_write,          /*ssize_t (*write)(struct file *,const char *,size_t,loff_t *);*/
                                          /*int (*readdir)(struct file *,void *,filldir_t);*/
                                          /* unsigned int (*poll) (struct file *,struct poll_table_struct *);*/
    ioctl:           X10_ioctl,          /* int (*ioctl) (struct inode *,struct file *,unsigned int,unsigned long );*/
                                           /* int (*mmap) (struct file *,struct vm_area_struct *);*/
    open:            X10_open,            /* int (*open) (struct inode *,struct file *);*/
                                           /* int (*flush)(struct file *);*/
    release:         X10_release,          /* int (*release)(struct inode *,struct file *);*/
                                           /* int (*fsync)(struct file *,struct dentry* ,int datasync);*/
                                           /* int (*fasync)(int,struct file *,int);*/
                                           /* int (*lock)(struct file *,int,struct file_lock *);*/
                                           /*ssize_t (*readv)(struct file *,const struct iovec*,unsigned long,loff_t*).*/
                                           /*ssize_t (*writev)(struct file *,const struct iovec*,unsigned long,loff_t*).*/
};

/*====================================================================
Each USB Pipe was an individual wait queue and flag this is to
lock a pipe so each pair of operations write then read appears effectively
atomic from user space
Each device context has an array of pipe stuctures
====================================================================*/
static void InitPipe( struct Pipe* pipe )
{
	init_waitqueue_head( &pipe->queue );
	pipe->InUse = 0;
	pipe->profile_data = NULL ;
}

static int read_pipe_profile_data( char * buf, char ** start, off_t offset, int count, int * eof, void * data )
{
	struct Pipe	* pipe = (struct Pipe	*)data ;
	struct X10ProfileEntry * Ptr = pipe->profile_data ;
	unsigned int x ;
	int len = 0;
	int Limit=count - 80 ;

	len +=sprintf(buf+len,"Opcode\t");
	for (x=0;x<NUMBER_OF_PROFILE_RANGES;x++)
	{
		len +=sprintf(buf+len,"%ld\t",ProfileRanges[x]);
		if (len>=Limit) goto end ;
	}
	len +=sprintf(buf+len,"\n");
	while( Ptr )
	{
		len +=sprintf(buf+len,"%d\t",Ptr->opCode);
		for (x=0;x<NUMBER_OF_PROFILE_RANGES;x++)
		{
			len +=sprintf(buf+len,"%ld\t",Ptr->Counts[x]);
		}
		len +=sprintf(buf+len,"\n");
		if (len>=Limit) goto end ;
		Ptr=Ptr->Next ;
	}

	end :
	*eof = 1;
	return len ;
}

/*====================================================================
For each X10 attached we have a driver context structure
this function initialises it
====================================================================*/
static void InitDriverContext( struct driver_context * context )
{
	int x;
	struct driver_context * ptr = ContextList;
	char name[32];
	context->DownloadFinished =0			;
	context->True		=	1				;
	context->False		=	0				;
	context->Number = 0;
	while( ptr )
	{
		if( ptr->Number == context->Number )
		{
			context->Number = ptr->Number + 1;
			ptr = ContextList;
			continue;
		}

		ptr = ptr->next;
	}
#ifdef _X10_USB_DEBUG
	printk( "X10 Minor Number = %d\n", context->Number );
#endif
	for( x=0; x<NUMBER_OF_PIPES; x++ )
	{
		InitPipe( &context->Pipes[x] );
		sprintf(name,"X10_%d_pipe_%d",context->Number,x);
		create_proc_read_entry(name,0444,0,read_pipe_profile_data,&context->Pipes[x]);
	}
}

static inline void timeval_less_local( struct timeval a, struct timeval b, struct timeval * result )
{
	if( a.tv_usec<b.tv_usec )
	{
		a.tv_sec--;
		a.tv_usec+= 1000000 ;
	}
	result->tv_sec=a.tv_sec-b.tv_sec ;
	result->tv_usec=a.tv_usec-b.tv_usec ;
}

/*====================================================================
This function performs most X10 operations write/read pairs from
the bulk pipes specified in the X10_COMMAND structure
====================================================================*/
static int DoX10IO( struct driver_context * context, X10_COMMAND *KernelCommandCopy )
{
	int CommandPipeIndex;
	int ReturnValue = 0;
	wait_queue_t wait;
	int Length;
#ifdef  _X10_USB_DEBUG
	int i, rxLength, logNumber;
	struct timeval start_time,end_time,delta_time ;
	unsigned long elapsed ;
#endif
	int OpCode ;
	int pipe_number ;
	int retry_count ;
#ifdef _X10_USB_DEBUG
	down( &TxHistory.LogSemaphore );
	logNumber = TxHistory.CurrentLog;

	TxHistory.CurrentLog = TxHistory.CurrentLog + 1;
	if ( TxHistory.CurrentLog >= NUM_TRANSACTIONS ) TxHistory.CurrentLog = 0;

	if ( TxHistory.UsedLogs >= NUM_TRANSACTIONS ) TxHistory.UsedLogs = NUM_TRANSACTIONS;
	else TxHistory.UsedLogs = TxHistory.UsedLogs + 1;
	up( &TxHistory.LogSemaphore );
#endif
	CommandPipeIndex = KernelCommandCopy->CommandpipeNum - 1;
	pipe_number=CommandPipeIndex ;
//	CommandPipeIndex = 0;
	init_waitqueue_entry( &wait, current );
	add_wait_queue( &context->Pipes[CommandPipeIndex].queue, &wait );

	while( 1 )
	{
		set_current_state( TASK_INTERRUPTIBLE );
		if( context->Pipes[CommandPipeIndex].InUse == 0 )
			break ;
		schedule();
	}

	set_current_state( TASK_RUNNING );
	remove_wait_queue( &context->Pipes[CommandPipeIndex].queue, &wait );
	context->Pipes[CommandPipeIndex].InUse = 1;
#ifdef _X10_USB_DEBUG
	for ( i = 0; i < KernelCommandCopy->CommandLength; i ++ )
	{
		TxHistory.LogEntries[logNumber].TxBuffer[i] = KernelCommandCopy->Buffer[i];
	}
	TxHistory.LogEntries[logNumber].PipeNum = KernelCommandCopy->CommandpipeNum;
	TxHistory.LogEntries[logNumber].TxTime = jiffies;
	TxHistory.LogEntries[logNumber].RxTime = 0;
	TxHistory.LogEntries[logNumber].TxLength = KernelCommandCopy->CommandLength;
	do_gettimeofday(&start_time);
#endif
	OpCode = KernelCommandCopy->Buffer[0];
	retry_count= 0 ;
	down( &context->USBCommunicationSemaphore );
	while (retry_count< MAX_USB_RETRIES)
	{
		if( usb_bulk_msg( 	context->dev,
							usb_sndbulkpipe( context->dev, KernelCommandCopy->CommandpipeNum ),
							&KernelCommandCopy->Buffer,
							KernelCommandCopy->CommandLength,
							&Length,
							HZ * TIMEOUT_MULT_HZ ) < 0 )
		{
#ifdef _X10_USB_DEBUG
			printk( "<1>X10 usb bulk write to  pipe %d  failed\n", KernelCommandCopy->CommandpipeNum );
#endif
			retry_count ++ ;
		}
		else
		{
			break ;
		}
	}
	up( &context->USBCommunicationSemaphore );

	if (retry_count >= MAX_USB_RETRIES)
	{
#ifdef _X10_USB_DEBUG
		printk( "<1>X10 retry count exceeded %d\n", KernelCommandCopy->CommandpipeNum );
		DisplayLogs( );
#endif
		ReturnValue = -ENOTTY;
		goto end ;
	}

	retry_count= 0 ;
	down( &context->USBCommunicationSemaphore );
	while (retry_count< MAX_USB_RETRIES)
	{
		if( usb_bulk_msg(	context->dev,
							usb_rcvbulkpipe( context->dev, KernelCommandCopy->AnswerpipeNum ),
							&KernelCommandCopy->Buffer,
							128,
							&Length,
							HZ * TIMEOUT_MULT_HZ ) < 0 )
		{
#ifdef _X10_USB_DEBUG
			printk( "<1>X10 usb bulk read from  pipe %d  failed\n", KernelCommandCopy->AnswerpipeNum );
#endif
			retry_count ++ ;
		}
		else
		{
#ifdef _X10_USB_DEBUG
			do_gettimeofday(&end_time);
			timeval_less_local(end_time,start_time,&delta_time);
			elapsed = delta_time.tv_usec / 1000 ;	//convert to millisec
			add_profile_entry(OpCode, &context->Pipes[pipe_number],elapsed);
			rxLength = Length;
			if ( rxLength > LOG_BUFFER_SIZE ) rxLength = LOG_BUFFER_SIZE;
			for ( i = 0; i < rxLength; i ++ )
			{
				TxHistory.LogEntries[logNumber].RxBuffer[i] = KernelCommandCopy->Buffer[i];
			}
			TxHistory.LogEntries[logNumber].RxTime = jiffies;
			TxHistory.LogEntries[logNumber].RxLength = rxLength;
#endif
			break ;
		}
	}
	up( &context->USBCommunicationSemaphore );

	if (retry_count >= MAX_USB_RETRIES)
	{
#ifdef _X10_USB_DEBUG
		printk( "<1>X10 retry count exceeded %d\n", KernelCommandCopy->CommandpipeNum );
		DisplayLogs( );
#endif
		ReturnValue = -ENOTTY;
		goto end ;
	}

	KernelCommandCopy->AnswerLength = Length;
end :
	context->Pipes[CommandPipeIndex].InUse = 0;
	wake_up( &context->Pipes[CommandPipeIndex].queue );

	return( ReturnValue );
}


/*====================================================================
Constructs and executes an X10 Command
====================================================================*/
static int MakeAndSendX10Command(	struct driver_context * context, unsigned char CommandCode,
									char * txBuffer, int txLength, int TxPipe,
									char* rxBuffer, int *rxLength, int RxPipe )
{
	int ReturnCode = 0;
	int ReturnLength;
	X10_COMMAND * Command = kmalloc( sizeof( X10_COMMAND ), GFP_KERNEL|__GFP_DMA );

	if( !Command )
	{
		printk( "Failed to allocate memory for X10 command\n" );
		return -ENOMEM ;
	}

	memset( Command, 0, sizeof( X10_COMMAND ) );
	Command->CommandpipeNum = TxPipe;
	Command->AnswerpipeNum = RxPipe;
	Command->CommandLength = txLength + 1;
	Command->AnswerLength = 64;
	Command->Buffer[0] = CommandCode;
	if( txBuffer && txLength ) memcpy( &Command->Buffer[1], txBuffer, txLength );

	ReturnCode = DoX10IO( context, Command );
	if( ReturnCode != -ENOTTY )
	{
		ReturnLength = *rxLength<Command->AnswerLength?*rxLength:Command->AnswerLength;
		if( rxBuffer && *rxLength )
			memcpy( rxBuffer, Command->Buffer, ReturnLength );
		if( rxBuffer[0] != CommandCode || rxBuffer[1] != USB_MESSAGE_EXECUTION_SUCCESS )
			ReturnCode = -ENOTTY;
	}

	kfree( Command );

	return( ReturnCode );
}

/*====================================================================
Checks to see if the X10 already has a valid set of software downloaded
====================================================================*/
static int DownloadDone( struct driver_context * context )
{
	unsigned char Message[10];
	int rxSize = sizeof( Message );
	int ReturnValue;

	ReturnValue = MakeAndSendX10Command(	context,
											USB_MESSAGE_IS_CODE_DOWNLOAD_COMPLETE,
											NULL,
											0,
											USB_PIPE_SRAM,
											Message,
											&rxSize,
											USB_PIPE_SRAM );
	if( ReturnValue < 0 )
	{
#ifdef _X10_USB_DEBUG
		printk( "X10 Download Done query failed\n" );
#endif
		return( ReturnValue );
	}

	return( Message[2] != 0 );
}

/*====================================================================
Signals to the X10 that a download into SRAM is about to start
====================================================================*/
static int SendDownloadStart( struct driver_context * context )
{
	unsigned char Message[10];
	int rxSize = sizeof( Message );
	int ReturnValue;

	ReturnValue = MakeAndSendX10Command(	context,
											USB_MESSAGE_CODE_DOWNLOAD_START,
											NULL,
											0,
											USB_PIPE_SRAM,
											Message,
											&rxSize,
											USB_PIPE_SRAM );
	if( ReturnValue < 0 )
	{
 #ifdef _X10_USB_DEBUG
		printk( "X10 Download Start failed\n" );
#endif
		return( ReturnValue );
	}

	return( Message[2] != 0 );
}


/*====================================================================
Signals to X10 that a SRAM download has finished
====================================================================*/
static int SendFinished( struct driver_context * context )
{
	unsigned char Message[10];
	int rxSize = sizeof( Message );
	int ReturnValue;

	ReturnValue = MakeAndSendX10Command(	context,
											USB_MESSAGE_CODE_DOWNLOAD_COMPLETED,
											NULL,
											0,
											USB_PIPE_SRAM,
											Message,
											&rxSize,
											USB_PIPE_SRAM );
	if( ReturnValue < 0 )
	{
#ifdef _X10_USB_DEBUG
		printk( "Download Finished Failed\n" );
#endif
		return( ReturnValue );
	}

	return( Message[2] != 0 );
}

#define SRAM_BUFFER_SIZE  64

/*====================================================================
Downloads the SRAM Code to the 8051
====================================================================*/
static int SendSRamData( struct driver_context * context )
{
	unsigned char LengthThisPacket;
	X10_COMMAND * Command;
	int ReturnCode;
	int lengthWritten = 0;
	int totalLength = USED_MEMORY;

	Command = kmalloc( sizeof( X10_COMMAND ), GFP_KERNEL|__GFP_DMA );
	if( !Command ) return( -ENOMEM );

	while( lengthWritten < USED_MEMORY )
	{
		LengthThisPacket = totalLength-lengthWritten< SRAM_BUFFER_SIZE-4?totalLength-lengthWritten:SRAM_BUFFER_SIZE-4;
		Command->Buffer[0] = USB_MSG_WRITE_SRAM;		//Command
		Command->Buffer[1] = LengthThisPacket;			//Length
		Command->Buffer[2] = lengthWritten & 0xFF;		//Address
		Command->Buffer[3] = (lengthWritten >> 8) & 0xFF;
		Command->CommandpipeNum = USB_PIPE_SRAM;
		Command->AnswerpipeNum = USB_PIPE_SRAM;
		memcpy( &Command->Buffer[4], &Memory8051[lengthWritten], LengthThisPacket );
		Command->CommandLength = LengthThisPacket + 4;
		Command->AnswerLength = 64;
		ReturnCode = DoX10IO( context, Command );
		if( ReturnCode < 0 )
		{
			kfree( Command );
			return( -ENOTTY );
		}
		lengthWritten += LengthThisPacket;
	}

	kfree( Command );

	return( 0 );
}


/*====================================================================
Performs a full sram download into the X10
====================================================================*/
static int SendSRam( struct driver_context * context )
{
	if( DownloadDone( context ) ) return( 1 );

#ifdef _X10_USB_DEBUG
	printk( "<1>X10 Firmware download started.\n" );
#endif

	if( SendDownloadStart( context ) < 0 ) return 0;
	if( SendSRamData( context ) < 0 ) return 0;
	if( SendFinished( context ) < 0 ) return 0;

#ifdef _X10_USB_DEBUG
	printk( "<1>X10 Firmware download completed.\n" );
#endif

	return( 1 );
}

/*====================================================================
Handles a request to perform X10 IO on behalf of a process in user space
Needs to copy user data from user space into kernel space do the operation
and copy the results back
====================================================================*/
static int DoUserX10IO( struct driver_context * context, X10_COMMAND *UserCommand )
{
	int ReturnValue = 0;
	X10_COMMAND * KernelCommandCopy;

	if( context == NULL )
	{
#ifdef _X10_USB_DEBUG
		printk( "DoUserX10IO passed invalid context\n" );
#endif
		return( -EFAULT );
	}
	if( context->dev == NULL )
	{
#ifdef _X10_USB_DEBUG
		printk( "X10 Context has no valid device\n" );
#endif
		return( -EFAULT );
	}

	KernelCommandCopy = (X10_COMMAND *)kmalloc( sizeof( X10_COMMAND ), GFP_KERNEL|__GFP_DMA );
	if( !KernelCommandCopy )
	{
#ifdef _X10_USB_DEBUG
		printk( "X10 kernel malloc failed\n" );
#endif
		return( -ENOMEM );
	}
	if( copy_from_user( KernelCommandCopy, UserCommand, sizeof( X10_COMMAND ) ) )
	{
#ifdef _X10_USB_DEBUG
		printk( "copy_from_user failed\n" );
#endif
		return( -EFAULT );
	}
	ReturnValue = DoX10IO( context, KernelCommandCopy );
	if( copy_to_user( UserCommand, KernelCommandCopy, sizeof( X10_COMMAND ) ) )
	{
#ifdef _X10_USB_DEBUG
		printk( "copy_to_user failed\n" );
#endif
		return( -EFAULT );
	}
	kfree( KernelCommandCopy );

	return( ReturnValue );
}

/*====================================================================
Adds new driver context into linked list
=====================================================================*/
static inline int link_new_driver_context( struct driver_context * new_context )
{
	new_context->next = ContextList;
	ContextList = new_context;

	return( 0 );
}

/*====================================================================
Walks Linked list and counts the number of X10's currently connected
=====================================================================*/
static inline int count_connected_X10( void )
{
	int count = 0;
	struct driver_context * ptr = ContextList;

	while( ptr )
	{
		count++;
		ptr = ptr->next;
	}

	return( count );
}

/*====================================================================
Removes an X10 from the master linked list
=====================================================================*/
static inline void unlink_driver_context( struct driver_context * context )
{
	struct driver_context ** ptr = &ContextList;

	while( *ptr )
	{
		if( *ptr == context )
		{
			*ptr = context->next;
			return;
		}
		ptr = &( *ptr )->next;
	}
}


/*====================================================================
Unlinks a context from linked list and frees its resources
=====================================================================*/
static void free_driver_resources( struct driver_context * context )
{
	char name[32];
	int x;

	unlink_driver_context( context );
	for( x=0; x<NUMBER_OF_PIPES; x++ )
	{
		sprintf(name,"X10_%d_pipe_%d",context->Number,x);
		remove_proc_entry(name,0);
#ifdef _X10_USB_DEBUG
		delete_profile_entries(&context->Pipes[x].profile_data);
#endif
	}
	kfree( context );
}


/*====================================================================
Allocates and initalises a driver context structure
=====================================================================*/
static  struct driver_context * allocate_x10_driver_resources()
{
	struct driver_context * ptr;
	ptr = (struct driver_context *)kmalloc( sizeof( struct driver_context ), GFP_KERNEL|__GFP_DMA );
	if( ptr )
	{
		InitDriverContext( ptr );
		link_new_driver_context( ptr );
	}
	return( ptr );
}

/*====================================================================
The driver uses the /proc file system to report currently connected
X10's This is the function that oitputs that data
Currently the attached devices minor numbers
=====================================================================*/
static int proc_read_procmem( char * buffer, char **start, off_t offset, int len, int *unused, void *data )
{
	#define LIMIT (PAGE_SIZE-80)
	struct driver_context * ptr = ContextList;

	len=0;
	while( ptr )
	{
		if( ptr->Type == IOCARD )
		{
			len += sprintf( buffer + len, "%d\n", ptr->Number );
		}
		ptr = ptr->next;
	}
	return( len );
}

/*====================================================================
Issues a reset command to the X10
=====================================================================*/
static int X10Reset( struct usb_device * dev,struct driver_context * context,  unsigned char Reset )
{
	int ReturnValue;
	unsigned char * Ptr ;

	Ptr = Reset?&context->True:&context->False ;
	ReturnValue = usb_control_msg( dev,
			/*pipe*/usb_sndctrlpipe( dev, 0 ),
			/*request*/ANCHOR_LOAD_INTERNAL,
			/*request Type*/USB_TYPE_VENDOR|USB_RECIP_DEVICE|USB_DIR_OUT, //host to device|vendor type|device
			/*message value*/CPUCS_REG,
			/*message Index*/0,
			/*Data Ptr*/Ptr,
			/*Data Size*/1,
			/*timeout*/HZ * 10
			);
	return( ReturnValue < 0 );
}


/*====================================================================
Sends an Intel Hex record via the control pipe to an X10
=====================================================================*/
static int SendHexRecordToControl( struct usb_device * dev, unsigned int Type, PINTEL_HEX_RECORD SourcePtr )
{
	PINTEL_HEX_RECORD Ptr;
	unsigned int NumberToDo = SourcePtr->Length;
	unsigned short int TargetAddress = SourcePtr->Address;
	unsigned int NumberThisTime;
	unsigned int NumberDone = 0;
	const unsigned int MaxToDo = 16;

	Ptr = kmalloc( sizeof( INTEL_HEX_RECORD ), GFP_KERNEL|__GFP_DMA );
	if ( !Ptr )
	{
		printk( "X10 kmalloc failed in %s Line %d\n", __FUNCTION__, __LINE__ );
		return 0;
	}

	memcpy( Ptr, SourcePtr, sizeof( INTEL_HEX_RECORD ) );
	while ( NumberToDo )
	{
		NumberThisTime = NumberToDo<MaxToDo?NumberToDo:MaxToDo;
		if ( usb_control_msg( dev, usb_sndctrlpipe( dev, 0 ), Type, USB_TYPE_VENDOR|USB_RECIP_DEVICE|USB_DIR_OUT, TargetAddress, 0, &Ptr->Data[NumberDone], NumberThisTime, HZ*5 ) < 0 )
		{
			printk( "Transfer to control Pipe zero Failed\n" );
			kfree( Ptr );
			return 0;
		}

		NumberDone += NumberThisTime;
		NumberToDo -= NumberThisTime;
		TargetAddress += NumberThisTime;
	}

	kfree( Ptr );
	return 1;
}

/*====================================================================
Downloads a block of Intel hex records to the X10
=====================================================================*/
static int DownloadIntelHex( struct usb_device * dev,struct driver_context * context, PINTEL_HEX_RECORD hexRecord )
{
	PINTEL_HEX_RECORD SrcPtr = hexRecord;
	int TransferCount = 0;
	int ReturnValue = 1;

	while( SrcPtr->Type == 0 )
	{
		if( !INTERNAL_RAM( SrcPtr->Address ) )
		{
			if( !SendHexRecordToControl( dev, ANCHOR_LOAD_EXTERNAL, SrcPtr ) )
			{
#ifdef _X10_USB_DEBUG
				printk( "ANCHOR_LOAD_EXTERNAL Failed to Send %d Bytes\n", SrcPtr->Length );
#endif
				ReturnValue = 0;
				goto end;
			}
			else
			{
				TransferCount += SrcPtr->Length;
			}
		}
		SrcPtr++;
	}

	X10Reset( dev,context, 1 );
	SrcPtr = hexRecord;
	TransferCount = 0;

	while( SrcPtr->Type == 0 )
	{
		if( INTERNAL_RAM( SrcPtr->Address ) )
		{
			if( !SendHexRecordToControl( dev, ANCHOR_LOAD_INTERNAL, SrcPtr ) )
			{
#ifdef _X10_USB_DEBUG
				printk( "ANCHOR_LOAD_INTERNAL Failed to send %d Bytes\n", SrcPtr->Length );
#endif
				ReturnValue = 0;
				goto end;
			}
			else
			{
				TransferCount += SrcPtr->Length;
			}
		}
		SrcPtr++;
	}

	end :
	return ReturnValue;
}

#if 0
/*====================================================================
A thread to look for input data packets returned from the X10
=====================================================================*/
static int x10_activity_thread(void * data)
{
        /*
         * This thread doesn't need any user-level access,
         * so get rid of all our resources
         */
	char * buffer ;
	char thread_name[64];
	int Length ;
	struct driver_context * context=data;
        daemonize();
        reparent_to_init();
		printk("<1> X10 Line%d\n",__LINE__);
	buffer = kmalloc(64, GFP_KERNEL|__GFP_DMA);
        /* Setup a nice name */
	sprintf(thread_name,"kx10_%02dactivityd",context->Number);
		printk("<1> X10 Line%d\n",__LINE__);
        strcpy(current->comm,thread_name );
        /* Send me a signal to get me die (for debugging) */
        do
		{
//			mdelay( 50 );
			down( &context->USBCommunicationSemaphore );
			if( usb_bulk_msg(	context->dev,
						usb_rcvbulkpipe( context->dev, BACK_CHANNEL_PIPE ),
						buffer,
						64,
						&Length,
						HZ * TIMEOUT_MULT_HZ *10 ) < 0 )
			{
				printk("<1> Thread timed out\n");
			}
			else
			{
				up(&context->InputBufferSemaphore);
				memcpy(context->LastIOPacket,buffer,Length)				;
				do_gettimeofday(&context->LastInputPacketTime);
				context->LastIOPacketSize=Length ;
				down(&context->InputBufferSemaphore);
			}
			up( &context->USBCommunicationSemaphore );
        } while (!signal_pending(current)&& context->dev);
        dbg("x10_activity_thread exiting");
        complete_and_exit(&context->kx10d_exited, 0);
		context->kx10d_pid = -1;
}
#endif
/*====================================================================
Called by the USB subsystem to see if we recognise any newly attached
USB devices. Use this as an opportunity to download code into X10
=====================================================================*/
#ifdef OLD_KERNEL
static void * X10_usb_probe( struct usb_device * dev, unsigned intf, const struct usb_device_id * id )
#else	//probe changes from 2.4 to 2.6
 static int X10_usb_probe(struct usb_interface * interface, const struct usb_device_id * id )
#endif
{
	struct driver_context * context;
#ifndef OLD_KERNEL
	struct usb_device * dev = interface_to_usbdev(interface) ;
#endif
	//pid_t pid ;
	if( dev->descriptor.idVendor == HEBER && dev->descriptor.idProduct== X10_LOADER )
	{
	#ifdef OLD_KERNEL
		MOD_INC_USE_COUNT ;	//not needed for 2.6
	#endif
#ifdef _X10_USB_DEBUG
		printk( "<1>X10 Loader board connected \n" );
#endif
		context = allocate_x10_driver_resources();
		context->Type = LOADER;

		sema_init( &context->USBCommunicationSemaphore, 1 );

		//
		// First download a stub loader
		//
		//
		X10Reset( dev,context, 1 );
#ifdef _X10_USB_DEBUG
		printk("Starting loader Download\n");
#endif
		if(!DownloadIntelHex( dev,context, loader ))
		{
			goto Error ;
		}
#ifdef _X10_USB_DEBUG
		printk("Finished loader Download\n");
#endif
		X10Reset( dev,context, 0 );
		//
		// Now download the device firmware
		//
		//
#ifdef _X10_USB_DEBUG
		printk("Starting firmware Download\n");
#endif
		if(!DownloadIntelHex( dev,context, firmware ))
		{
			goto Error ;
		}
#ifdef _X10_USB_DEBUG
		printk("Finishing firmware Download\n");
#endif
		//
		//	Issue a reset to the X10 on comming
		//  out of reset the X10 should disconnect and
		//  reconnect as an X10 with a different product
		//  descripter
		X10Reset( dev,context, 1 );
		X10Reset( dev,context, 0 );
	#ifdef OLD_KERNEL
		return( context );	//probe return changes from 2.4 to 2.6
	#else
		dev_set_drvdata(&interface->dev,context);
		return 0 ;
	#endif
	}
	if( dev->descriptor.idVendor == HEBER && dev->descriptor.idProduct == X10 )
	{
		//MOD_INC_USE_COUNT; 		2.6 change
	 #ifdef _X10_USB_DEBUG
		printk( "<1>X10 board connected \n" );
	 #endif
		context = allocate_x10_driver_resources();
		context->Type = IOCARD;
		context->dev = dev;
		mdelay(10);
	#ifdef OLD_KERNEL	//2.6 assigns drivers by interface/configuration rather than device
		if( usb_set_configuration( dev, 1 ) )
		{
#ifdef _X10_USB_DEBUG
			printk( "<1>Failed to set X10 to configuration 1\n" );
#endif
			goto Error;
		}
		else
		{
#ifdef _X10_USB_DEBUG
			printk( "<1>Set X10 to configuration 1\n" );
#endif
		}

		if( usb_set_interface( dev, 0, 0 ) )
		{
#ifdef _X10_USB_DEBUG
			printk( "<1>Failed to set X10 to Interface 0\n" );
#endif
			goto Error;
		}
		else
		{
#ifdef _X10_USB_DEBUG
			printk( "<1>Set X10 to Interface 0\n" );
#endif
		}
 #endif		//OLD_KERNEL

		if(SendSRam( context ))
			context->DownloadFinished = 1 ;
		printk( "<1>Firefly X10 Board ready.\n" );
		sema_init(&context->InputBufferSemaphore,1);
		/*
		init_completion(&context->kx10d_exited);
		context->LastIOPacketSize=0;
		pid = kernel_thread(x10_activity_thread,context,CLONE_FS|CLONE_FILES|CLONE_SIGHAND);
		if (pid<0)
		{
			printk( "<1>Firefly X10 Monitor thread creation failed.\n" );
		}
		context->kx10d_pid = pid ;
		*/

	#ifdef OLD_KERNEL
		return context; // probe return value changes from 2.4 to 2.6
	#else
		dev_set_drvdata(&interface->dev,context);
		return 0 ;
	#endif
	}
	#ifdef OLD_KERNEL
	return( NULL );
	#else
	return -ENODEV ;
	#endif
	Error:
	free_driver_resources( context );
	#ifdef OLD_KERNEL  // probe return value changes from 2.4 to 2.6
		MOD_DEC_USE_COUNT;
		return( NULL );
	#else
		return -ENOMEM ;
	#endif
}

/*====================================================================
Handle a sudden X10 disconnext
=====================================================================*/
	#ifdef OLD_KERNEL
static void X10_usb_disconnect( struct usb_device * usbdev, void * context_ptr )
	#else
static void X10_usb_disconnect( struct usb_interface * interface )
	#endif
{
	#ifdef OLD_KERNEL		//disconnect callback changes from 2.4 to 2.6
	struct driver_context * context = context_ptr;
	#else
	struct driver_context * context = dev_get_drvdata(&interface->dev);
	#endif

	/*context->remove_pending = 1;
	wake_up( &context_wait );
	sleep_on( &context->remove_ok ); */
	/*
	if (context->Type == IOCARD &&context->kx10d_pid>=0) 	// If thread OK then kill it
	{
		kill_proc(context->kx10d_pid, SIGTERM, 1);
		wait_for_completion(&context->kx10d_exited);
	}
	*/
#ifdef _X10_USB_DEBUG
	if( context->Type == LOADER )
		printk( "<1> Freeing X10 Loader resources\n" );
	else
		printk( "<1> X10 Freeing driver resources\n" );
#endif
	down( &context->USBCommunicationSemaphore );
	free_driver_resources( context );
	up( &context->USBCommunicationSemaphore );
	#ifdef OLD_KERNEL		//not used by 2.6 kernels
	MOD_DEC_USE_COUNT ;
	#endif
}

/*====================================================================
List of device ID's we want to handle
=====================================================================*/
static struct usb_device_id X10_usb_ids[] ;
static struct usb_device_id X10_usb_ids [] =
{
	{USB_DEVICE(HEBER ,X10) },
	{USB_DEVICE(HEBER ,X10_LOADER) },
	{ },
};

MODULE_DEVICE_TABLE (usb, X10_usb_ids);

/*====================================================================
structure to pass to USB subsystem
=====================================================================*/
static  struct usb_driver X10_driver_def = {
    name:               "X10",
    probe:              X10_usb_probe,
    disconnect:         X10_usb_disconnect,
    //driver_list :       {NULL,NULL},
    //minor:              160  ,
    id_table:           X10_usb_ids ,
    } ;

/*====================================================================
	Defines a name for this driver
=====================================================================*/
static char * X10_name = "X10";

/*====================================================================
Called when X10 module is first loaded
=====================================================================*/
static int __init X10_init_module( void )
{
	int result;

#ifdef _X10_USB_DEBUG
	printk( "<1>Initialising log semaphore\n" );
	sema_init( &TxHistory.LogSemaphore, 1 );
#endif

	if( ( result = register_chrdev( X10_major_number, X10_name, &X10_file_operations ) ) < 0 )
	{
#ifdef _X10_USB_DEBUG
		printk( "<1> Failed to register X10 char device returned %x\n", result );
#endif
		return( result );
	}

	/* major number was dynamically allocated */
	if( X10_major_number == 0 ) X10_major_number = result;
	if ( ( result = usb_register( &X10_driver_def ) ) < 0 )
	{
#ifdef _X10_USB_DEBUG
		printk( "<1> Failed to register X10 driver register returned %x\n", result );
#endif
		return( -1 );
	}

	create_proc_read_entry( "driver/X10", 0, NULL, proc_read_procmem, NULL );

	return( 0 );
}

/*====================================================================
Called when X10 module is unloaded
=====================================================================*/
static void __exit X10_cleanup_module( void )
{
	remove_proc_entry( "driver/X10", NULL );
	unregister_chrdev( X10_major_number, X10_name );
	usb_deregister( &X10_driver_def );
	printk( "<1>%s UnLoaded\n", TITLE );
}

module_init(X10_init_module);
module_exit(X10_cleanup_module);

