/***************************************************************************
*
*     Project:      Firefly X10 Board
*
*     File:         fflyusb.h
*
*     Version:      1.19
*
*     Description:  FireFly X10 API
*
*     Notes:
*
*     Changes:
*     1.19 MJB      Added CachedReadAndResetSecuritySwitchFlags(), 15/2/2005
*     1.18 MJB      Added new function CheckEEPROM(). This detects
*                   the currently fitted EEPROM. It can differentiate
*                   between the following devices: 24LC01, 24LC02, 24LC04,
*                   24LC08, 24LC16, 24LC32, 24LC64, 24LC128, 24LC256 and
*                   24LC512, 29/9/2004
*     1.17 MJB      - Added new functions ConfigureReelsEx() and
*                     GetReelStatusEx(). These store the number of half
*                     steps per reel, the reel position and reel error as
*                     WORDS instead of BYTES. This allows for 200 step
*                     reels. The old functions ConfigureReels() and
*                     GetReelStatus() are now obsolete and should not
*                     be used.
*                   - Added new function SetDutyCycle(). This allows
*                     the reel on/off duty cycle to be programmed, 11/2/2004
*     1.16 MJB      The FireflyUSB class has changed and now contains
*                   a private pointer to a set of instance data. The
*                   driver can store local data such as the device handle
*                   in this structure. This was done to help support
*                   using multiple X10's concurrently, while keeping the
*                   FireflyUSB class as stable as possible, 10/7/2003
*     1.15 MJB      - Added PulseOffOutput() and ConfigureEEPROM().
*                   - Added include file 'x10eeprom.h'.
*                   - Changed/added/removed some private member
*                     functions/variables, 26/6/2003
*     1.14 MJB      Added GetProductVersion(), 17/3/2003
*     1.13 MJB      - Added new GetParityErrors() function.
*                   - Removed all IO/Serial/Error code definitions as they
*                     are now defined in separate include files 'x10io.h',
*                     'x10serl.h' and 'x10err.h', 14/3/2003
*     1.12 MJB      Removed necessity to specify upper pulse time length
*                   in parallel input reading, 19/02/2003
*     1.11 MJB      Added new poll_retry_count variable to cctalk mode 1
*                   and polled RS232 sturctures, 30/01/2003
*     1.10 MJB      Changed polled RS232 and cctalk mode 1 maximum timeout
*                   parameter from BYTE to WORD in order to achieve longer
*                   timeouts (upto 65 seconds), 12/12/2002
*     1.9 MJB       Added new trigger method for cctalk and RS232
*                   polling methods, 22/11/2002
*     1.8 MJB       - Ensured all function parameters follow the same
*                     standard.
*                   - Added Parallel IO functions, 20/11/2002
*     1.7 MJB       - Added RS232 Polled Mode.
*                   - Renamed enum CCTalkPollMethod to PollMethod as
*                     it is used in cctalk mode 1 and rs232 polled mode.
*                   - Also renamed functions EmptyCCTalkBuffer,
*                     ReceiveCCTalkMessage and DeleteCCTalkMessage to
*                     EmptyPolledBuffer, ReceivePolledBuffer and
*                     DeletePolledBuffer respectively.
*                   - When configuring cctalk mode 1, removed requirement
*                     to specify device address and poll and inhibit
*                     message length, as these are already included
*                     in the message, 14/10/2002
*     1.6 MJB       - Corrected revision number history
*                   - Added new error code USB_MESSAGE_PIPE_UNAVAILABLE
*                   - Removed private variables from header file:
*                     deviceHandle, usbLastError, OutOnCopy,
*                     OutOffCopy, 12/9/2002
*     1.5 MJB       Changed maximum CCTalk mode 1 poll and inhibit message
*                   length to 25, and is defined in
*                   MAX_CCTALK_MODE1_MSG_LENGTH, 16/8/2002.
*     1.4 MJB       Added error code USB_MESSAGE_UNKNOWN_SERIAL_PROTOCOL.
*                   Added PORT_CCTALK_MODE1 to enum usbPortType.
*                   Created CCTalk Mode 1 structures CCTalkPollMethod
*                   and CCTalkConfig.
*                   Added CCTalk Mode 1 function declarations:
*                   - ConfigureCCTalkPort()
*                   - EmptyCCTalkBuffer()
*                   - ReceiveCCTalkMessage()
*                   - DeleteCCTalkMessage(), 5/8/2002
*     1.3 MJB       Tidied formatting. Removed redundant definitions.
*                   Corrected some comments. Added function
*                   declarations GetChangedInputs, ModifyOutputs,
*					PulseOutput, PulseOutputResult. 6/6/2002.
*     1.2 MJB		Removed erroneous ezloader.rc. 22/5/2002.
*     1.1 MJB		Added SPI functions. 22/5/2002
*     1.0 MJB		Major changes.
*					- Created object class for all functions.
*					- Added software version functions for DLL,
*					  8051 and PIC.
*					- Added proper error return codes.
*					- Many other minor changes.
*     0.0 PJS       First version, 28/8/2001
*
***************************************************************************/

/********    Header Sentry    ********/
#ifndef FIREFLYUSB_H
#define FIREFLYUSB_H

#include "x10types.h"
#include "x10io.h"				// Input/Output/Parallel definitions
#include "x10serl.h"			// Serial: RS232 and cctalk definitions
#include "x10eeprom.h"			// EEPROM definitions
#include "x10err.h"				// Error code definitions

/********    Types    ********/

#ifndef LPUINT
#define LPUINT	long unsigned int *
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////
//     FIREFLY API                                                                                //
////////////////////////////////////////////////////////////////////////////////////////////////////
class X10IOLock ; 
class X10Implementation ;
class FireFlyUSB
{
public:
	//** Required class functions *************************************************
	FireFlyUSB( );					// Constructor
	~FireFlyUSB( );					// Destructor

	//** General functions ********************************************************
	BOOL InitUSBBoard( BYTE boardNumber );
	BOOL init( int PortNumber=0);					// Initialises USB
	BOOL close( );					// Closes handle to USB

	usbErrorCode GetLastError( );	// Returns last error code
	BOOL ClearErrors( );			// Clears last error code


	//** Software version information *********************************************
	BOOL GetProductVersion( LPBYTE versionProduct );
	BOOL GetDllVersion( LPBYTE versionDll );
	BOOL Get8051Version( LPBYTE version8051 ); 			// Memory pipe
	BOOL GetPICVersion( LPBYTE versionPIC );			// Security pipe

	BOOL GetPICSerialNumber( LPBYTE serialNumberPIC );	// Security pipe
	BOOL SetPICSerialNumber( LPBYTE serialNumberPIC );	// Security pipe


	//** IO ***********************************************************************
	// All of the following functions use the IO pipe
	// Obsolete functions
	BOOL GetInputBit( usbInputBitId input_bit_id, LPBOOL result );
	BOOL GetInputs( LPBYTE inputs );
	BOOL SetOutputBit( usbOutputBitId output_bit_id, BOOL bit_state );
	BOOL SetOutputs( LPBYTE outputs );

	// New functions - use these
	BOOL GetInputs( usbInput *inputs );
    BOOL GetMultiplexedInputs( usbMultiplexedInput *inputs );
    BOOL InputMultiplexing( usbInputMultiplexing input  );
	BOOL GetChangedInputs( usbInput *changedInputs );
	BOOL GetRawInputs( usbInput *rawInputs );
	BOOL ModifyOutputs( usbOutput *offOutputs, usbOutput *onOutputs );
	BOOL SetOnPeriodOutputs( usbOutput onPeriods );
	BOOL SetOffPeriodOutputs( usbOutput offPeriods );
	BOOL SetOnPeriodOutputBit( usbOutputId outputID, int bitNumber, BOOL bitState );
	BOOL SetOffPeriodOutputBit( usbOutputId outputID, int bitNumber, BOOL bitState );
	BOOL SetOutputBrightness( BYTE brightness );
	BOOL PulseOutput( BYTE outputNumber, BYTE pulseDurationMs );
	BOOL PulseOffOutput( BYTE outputNumber, BYTE pulseDurationMs );
	BOOL PulseOutputResult( LPBYTE timeRemaining, BOOL *pulseComplete, BOOL *currentDetected );

	// Parallel Interface
	BOOL ConfigurePulsedInput( usbInputBitId inputBitID, BYTE pulseLowerTime, ActiveState inputActiveState );
	BOOL BeginPulsedInputCheck( usbInputBitId inputBitID );
	BOOL EndPulsedInputCheck( usbInputBitId inputBitID );
	BOOL ResetPulsedInputCounter( usbInputBitId inputBitID );
	BOOL DecrementPulsedInputCounter( usbInputBitId inputBitID );
	BOOL ReadPulsedInputCounter( usbInputBitId inputBitID, BYTE *inputCounterValue );
	BOOL ReleaseParallelHopperCoins( usbInputBitId inputBit, BYTE pulseLowerTime, ActiveState inputActiveState,
									 usbOutputBitId outputBit, WORD coinTimeout, BYTE coinsToRelease );
	BOOL GetParallelHopperStatus( usbInputBitId inputBitID, ReleaseHopperCoinsStatus *status, BYTE *coinsReleased );

	// Reel Interface
	BOOL ConfigureReels( BYTE numberOfReels, BYTE halfStepsPerTurn, BYTE stepsPerSymbol );
	BOOL ConfigureReelsEx( BYTE numberOfReels, WORD halfStepsPerTurn, BYTE stepsPerSymbol );
	BOOL SpinReels( BYTE reelNumber, BYTE directionAndStepSize, WORD steps );
	BOOL SpinRampUp( BYTE reelNumber, usbRampTable rampUpTable );
	BOOL SpinRampDown( BYTE reelNumber, usbRampTable rampDownTable );
	BOOL SetDutyCycle( BYTE reelNumber, BYTE offPeriod, BYTE onPeriod );
	BOOL GetReelStatus( usbReelStatus *status );
	BOOL GetReelStatusEx( ReelStatusEx *status );
	BOOL ReelSynchroniseEnable( BYTE reelNumber );

	//** Serial *******************************************************************
	// Depending on the port used, the following functions can use either the
	// SERIAL_A or SERIAL_B pipe
	BOOL SetConfig( usbSerialPort port, LPDCB config, usbPortType type );
	BOOL Send( usbSerialPort port, LPBYTE data, UINT length );
	BOOL Receive( usbSerialPort port, LPBYTE data, LPUINT length );
	BOOL ReceiveByteWithTimestamp( usbSerialPort port, LPBYTE rxByte, LPBYTE interval, BOOL *received );
	BOOL SetTimeoutMessage( usbSerialPort port, LPBYTE message, BYTE numBytes, float timeout );
	BOOL GetParityErrors( usbSerialPort port, LPWORD parityErrors );

	// CCTalk Mode 1 and RS232 Polled Mode
	BOOL ConfigureCCTalkPort( usbSerialPort port, CCTalkConfig *cctalkConfig );
	BOOL ConfigureRS232Poll( usbSerialPort port, RS232PollConfig *pollConfig );
	BOOL EmptyPolledBuffer( usbSerialPort port, BYTE deviceNumber );
	BOOL ReceivePolledMessage( usbSerialPort port, BYTE deviceNumber, LPBYTE data, LPUINT length, BOOL *inhibited );
	BOOL DeletePolledMessage( usbSerialPort port, BYTE deviceNumber );

	//** SPI (Serial Peripheral Inferface) ****************************************
	// All of the following functions use the SPI pipe
	BOOL EnableSPI( void );
	BOOL DisableSPI( void );
	BOOL SendSPI( BYTE numberOfTxBits, LPBYTE txMessage, BYTE waitTimeMs, BYTE numberOfRxBits, LPBYTE rxMessage );
	BOOL SendSEC( BYTE command, BYTE id, BYTE numberOfTxBytes, LPBYTE txMessage,
                  BYTE waitTimeMs, BYTE numberOfRxBytes, LPBYTE rxMessage );


	//** EEPROM *******************************************************************
	// All of the following functions use the MEMORY pipe
	BOOL ConfigureEEPROM( X10EEPROM *eepromConfig );
	BOOL CheckEEPROM( X10EEPROM *eepromConfig );
	BOOL ReadEEPROM( WORD address, LPBYTE data, UINT totalLength );
	BOOL WriteEEPROM( WORD address, LPBYTE data, UINT totalLength );


	//** SRAM *********************************************************************
	// All of the following functions use the SRAM pipe
	BOOL ReadSRAM( WORD address, LPBYTE data, UINT totalLength );
	BOOL WriteSRAM( WORD address, LPBYTE data, UINT totalLength );


	//** Security: clock **********************************************************
	// All of the following functions use the SECURITY pipe
	BOOL SetClock( DWORD time );
	BOOL GetClock( LPDWORD time );


	//** Security: switches *******************************************************
	// All of the following functions use the SECURITY pipe
	BOOL NextSecuritySwitchRead( LPDWORD time, LPBYTE switches );
	BOOL StartSecuritySwitchRead( );
	BOOL ClearSecuritySwitches( );
	BOOL ReadAndResetSecuritySwitchFlags( LPBYTE closedSwitches, LPBYTE openSwitches );
	BOOL CachedReadAndResetSecuritySwitchFlags( LPBYTE closedSwitches, LPBYTE openSwitches );

private:
	BOOL PerformUSBIO( BYTE command,	  // Command, i.e. USB_MSG_GET_8051_VERSION
                       int pipeNum,       // Pipe number, i.e. USB_PIPE_MEMORY
                       LPBYTE txBuffer,	  // Out buffer
					   DWORD txLength,	  // Out buffer length
					   LPBYTE rxBuffer,	  // In buffer
					   DWORD *rxLength ); // In buffer length

	BOOL InitPort( int portNumber );
	BOOL UncheckedReadEEPROM( WORD address, LPBYTE data, UINT totalLength );
	BOOL Download8051Software( );
	X10Implementation * X10;
	friend class X10IOLock ;
};

#endif



