/*****************************************************************
*
*     Project:      Firefly X10 Board
*
*     File:         cctalkdemo.cpp
*
*     Version:      1.4
*
*     Description:  CCTalk Mode 1 Demonstration
*
*     Notes:        Console application to demonstrate CCTalk
*                   Mode 1 operation
*
*     Changes:
*     1.4 MJB       Added ability to reconfigure cctalk device
*                   by pressing 'I'. Added new poll retry
*                   parameter as implemented in issue 21 X10
*                   Windows DLL, 30/01/2003
*     1.3 MJB       Added NO_TRIGGER parameter when configuring
*                   cctalk mode 1. Increased timeout to 2
*                   seconds, 12/12/2002
*     1.2 MJB       Due to changes in the driver, have removed
*                   the specification of device address, poll
*                   and inhibit message lengths during cctalk
*                   configuration. Also renamed calls to
*                   ReceiveCCTalkMessage, DeleteCCTalkMessage
*                   and EmptyCCTalkBuffer to ReceivePolledMessage,
*                   DeletePolledMessage and EmptyPolledBuffer
*                   respectively, 14/10/2002.
*     1.1 MJB       Added 'break' to ReceiveCCTalkMessage and
*                   DeleteCCTalkMessage calls to ensure that if
*                   an error occurs it returns, 29/8/2002.
*     1.0 MJB       First version, 5/8/2002
*
*****************************************************************/

#include <stdio.h>
#ifndef X10_LINUX_BUILD
#include <conio.h>
#else
#include <ctype.h>
#include "x10types.h"
#include "linuxkb.c"
#endif
#include "fflyusb.h"
#include "unlockio.h"
#include "x10errhd.c"

#define MAX_CCTALK_RX_LEN		257				// Maximum CCTalk response length

int main( int argc, char* argv[] )
{
	FireFlyUSB FireFly;
	DCB config;									// Configure device
	usbSerialPort portNum;						// Port: PORT_A or PORT_B
	CCTalkConfig cctalk_config;					// CCTalk configuration structure
	BYTE rxMsg [MAX_CCTALK_RX_LEN + 1];			// Create buffer for message to receive
	BOOL is_inhibited;							// Is device inhibited?
	BOOL had_response;							// Have we received CCTalk response?
	unsigned long bytesRxd, i;
	char key;

	printf( "Firefly X10 Board\n" );
	printf( "=================\n\n" );
	printf( "CCTalk Mode 1 Demonstration\n\n" );

	printf( "\nInitialising and unlocking X10 device..." );
	// Get a handle to the device
	if ( !FireFly.init( ) )
		return( ExitFunction( "X10 initialisation failed.", &FireFly, 1 ) );
	UnlockX10( &FireFly );
	printf( "success.\n\n" );

	#ifdef X10_LINUX_BUILD
	InitialiseConsole( );
	EnableGetchNewline( );
	#endif

	while( 1 != 2 )
	{
		printf( "Use PORT A or PORT B (A or B)? " );
		key = getch();
		if ( key == 'A' || key == 'a' )      { portNum = PORT_A; break; }
		else if ( key == 'B' || key == 'b' ) { portNum = PORT_B; break; }
		else printf( "%c - Invalid port.\n", toupper( key ) );
	}
	printf( "%c\n", toupper( key ) );

	config.BaudRate = 9600;
	config.fOutxCtsFlow = FALSE;
	config.fRtsControl = RTS_CONTROL_TOGGLE;
	if ( !FireFly.SetConfig( portNum, &config, PORT_CCTALK_MODE1 ) )
		return( ExitFunction( "SetConfig failed.", &FireFly, 1 ) );

	// Configure CCTalk Port
	cctalk_config.device_number = 0;
	cctalk_config.method = Repeated;
	cctalk_config.next_trigger_device = NO_TRIGGER;
	cctalk_config.poll_retry_count = 5;
	cctalk_config.polling_interval = 200;
	cctalk_config.max_response_time = 2000;
	cctalk_config.min_buffer_space = 20;
	cctalk_config.poll_msg[0] = 0x02;
	cctalk_config.poll_msg[1] = 0x00;
	cctalk_config.poll_msg[2] = 0x01;
	cctalk_config.poll_msg[3] = 0xe5;
	cctalk_config.poll_msg[4] = 0x18;
	cctalk_config.inhibit_msg[0] = 0x02;
	cctalk_config.inhibit_msg[1] = 0x00;
	cctalk_config.inhibit_msg[2] = 0x01;
	cctalk_config.inhibit_msg[3] = 0xe5;
	cctalk_config.inhibit_msg[4] = 0x18;

	if ( !FireFly.ConfigureCCTalkPort( portNum, &cctalk_config ) )
		return( ExitFunction( "ConfigureCCTalkPort failed.", &FireFly, 1 ) );

	printf( "\nTesting CCTalk operation ('I' to reinitialise, 'R' to read, 'E' to erase buffer and 'C' to exit)\n\n" );

	while ( TRUE )
	{
		key = getch();

		if ( ( key == 'i' ) || ( key == 'I' ) )
		{
			printf( "Configure CCTalk Port...\n" );
			if ( !FireFly.ConfigureCCTalkPort( portNum, &cctalk_config ) )
				return( ExitFunction( "ConfigureCCTalkPort failed.", &FireFly, 1 ) );
		}
		else if ( ( key == 'r' ) || ( key == 'R' ) )
		{
			had_response = FALSE;

			do
			{
				// Receive data
				if ( !FireFly.ReceivePolledMessage( portNum, 0, rxMsg, &bytesRxd, &is_inhibited ) )
					return( ExitFunction( "ReceivePolledMessage failed.", &FireFly, 1 ) );

				// Print received data
				if ( bytesRxd > 0 )
				{
					had_response = TRUE;

					printf( "CCTalk Response = [ " );
					for ( i=0; i<bytesRxd; i++ ) printf( "%02X ", rxMsg[i] );
					printf( "]\n" );

					// Delete message
					if ( !FireFly.DeletePolledMessage( portNum, 0 ) )
						return( ExitFunction( "DeletePolledMessage failed.", &FireFly, 1 ) );
				}
				else
				{
					if ( !had_response ) printf( "No Response. " );

					if ( is_inhibited ) printf( "Device IS inhibited.\n" );
					else printf( "Device is NOT inhibited.\n" );
				}
			} while ( bytesRxd > 0 );
			printf( "\n" );
		}

		else if ( ( key == 'e' ) || ( key == 'E' ) )
		{
			printf( "Erase CCTalk Receive Buffer ... " );
			if ( !FireFly.EmptyPolledBuffer( portNum, 0 ) )
				return( ExitFunction( "EmptyPolledBuffer failed.", &FireFly, 1 ) );
			else printf( "success.\n" );
		}

		else if ( ( key == 'c' ) || ( key == 'C' ) ) break;
	}

	return( ExitFunction( "Leaving program.", &FireFly, 0 ) );
}

