/*****************************************************************
*
*     Project:      Firefly X10 Board
*
*     File:         iodemo.cpp
*
*     Version:      1.2
*
*     Description:  IO Demonstration
*
*     Notes:        Console application to demonstrate IO
*
*					Application prints out detailed information
*					about its operation on startup.
*
*     Changes:
*     1.2 MJB       Added menu. Implemented testing of output
*                   pulsing and input changing. Removed
*                   GetRawInputs test. 6/6/2002
*     1.1 MJB       Demonstrates GetRawInputs. Fixed code to only
*                   output to 6 bits of AUX output. Removed any
*                   dynamic memory allocation. Code tidied. 19/4/2002
*     1.0 MJB       First version, 1/2/2002
*
*****************************************************************/


#include <stdio.h>
#ifndef X10_LINUX_BUILD
#include <conio.h>
#else
#include <ctype.h>
#include "linuxkb.c"
#endif
#include <time.h>
#include "fflyusb.h"
#include "unlockio.h"
#include "x10errhd.c"

#define PULSE_DURATION_MS			150

void test_general_io( FireFlyUSB * FireFly );
void test_output_pulsing( FireFlyUSB * FireFly );
void test_input_change( FireFlyUSB * FireFly );

int main(int argc, char* argv[])
{
	FireFlyUSB FireFly;
	char key;

	printf( "Firefly X10 Board\n" );
	printf( "=================\n\n" );
	printf( "This application demonstrations X10 IO.\n\n" );

	// Get a handle to the device
	printf( "Establishing link with FireFly USB device..." );
	if ( !FireFly.init( ) )
		return( ExitFunction( "Initialisation error.", &FireFly, 1 ) );
	UnlockX10( &FireFly );
	printf( "success.\n" );

	#ifdef X10_LINUX_BUILD
	InitialiseConsole( );
	#endif

	while ( TRUE )
	{
		printf( "X10 IO Menu\n" );
		printf( "===========\n\n" );

		printf( "Options:\n" );
		printf( "    'T' to test general X10 input reading and output writing.\n" );
		printf( "    'P' to test output pulsing.\n" );
		printf( "    'U' to test input change.\n" );
		printf( "or, 'C' to exit.\n\n" );

		printf( "Please make a selection: " );

		key = toupper( getch() );
		printf( "\n\n" );
		if ( key == 'T' ) test_general_io( &FireFly );
		else if ( key == 'P' ) test_output_pulsing( &FireFly );
		else if ( key == 'U' ) test_input_change( &FireFly );
		else if ( key == 'C' ) break;
	}

	return( ExitFunction( "Leaving program.", &FireFly, 0 ) );
}

void test_general_io( FireFlyUSB * FireFly )
{
	BYTE inputs[USB_IN_LENGTH], outputs[USB_OUT_LENGTH];
	char key;
	int i;

	for ( i = 1; i < USB_OUT_LENGTH; i++ ) outputs[i] = 0x00;
	outputs[0] = 0x01;

	while ( TRUE )
	{
		if ( !FireFly->GetInputs( inputs ) )
		{
			printf( "\nFailure %d getting inputs.\n", (int)FireFly->GetLastError( ) );
			break;
		}

		/* Print inputs to screen. */
		printf( "\nInputs   0    1    2    SW   CS\n         " );
		for ( i = 0; i < USB_IN_LENGTH; i++ ) printf( "%02X   ", inputs[i] & 0x000000FF );

		if ( !FireFly->SetOutputs( outputs ) )
		{
			printf( "Failure %d setting outputs.\n", (int)FireFly->GetLastError( ) );
			break;
		}
		printf( "\n\nOutputs  0    1    2    3    AUX\n         " );
		for ( i = 0; i < USB_OUT_LENGTH; i++ ) printf( "%02X   ", outputs[i] & 0x000000FF );
		
		/* Toggle outputs. */
		for ( i = 0; i < USB_OUT_LENGTH; i++ )
		{
			if ( ( outputs[i] == 0x80 ) || ( ( i == 4 ) && ( outputs[i] == 0x20 ) ) )	// AUX only has 6 outputs
			{
				outputs[i] = 0x00;
 				outputs[(i != USB_OUT_LENGTH - 1 ? i + 1 : 0)] = 0x01;
				break;
			}
			else outputs[i] = outputs[i] * 2;
		}

		printf( "\n\nPress any key to update display or 'C' to close.\n" );

		key = getch();
		if ( key == 'C' || key == 'c' ) break;
	}
}

void test_output_pulsing( FireFlyUSB * FireFly )
{
	int output_bit_id = OUTPUT_BIT_OP0;
	BOOL bit_state = TRUE, bPulseComplete, bCurrentDetected;
	BYTE byTimeRemaining;
	char key;

	while ( TRUE )
	{
		printf( "Performing %dms pulse to output ", PULSE_DURATION_MS );
		if ( output_bit_id < 32 ) printf( "OP%d ... ", output_bit_id );
		else printf( "AUX%d ... ", (output_bit_id-32) );

		if ( !FireFly->PulseOutput( output_bit_id, PULSE_DURATION_MS ) )
		{
			printf( "\nFailure %d pulsing output bit.\n", (int)FireFly->GetLastError( ) );
			break;
		}

		do
		{
			if ( !FireFly->PulseOutputResult( &byTimeRemaining, &bPulseComplete, &bCurrentDetected ) )
			{
				printf( "\nFailure %d obtaining pulse output result.\n", (int)FireFly->GetLastError( ) );
				break;
			}
		} while ( !bPulseComplete );

		if ( bCurrentDetected ) printf( "current detected.\n" );
		else printf( "no current detected.\n" );

		output_bit_id++;
		if ( output_bit_id > OUTPUT_BIT_AUX5 )
		{
			output_bit_id = OUTPUT_BIT_OP0;
			if ( bit_state == TRUE ) bit_state = FALSE;
			else bit_state = TRUE;
		}

		printf( "\nPress any key to update display or 'C' to close.\n" );

		key = getch();
		if ( key == 'C' || key == 'c' ) break;
	}
}

void test_input_change( FireFlyUSB * FireFly )
{
	BYTE inputs[USB_IN_LENGTH];
	usbInput changedInputs;
	char key;
	int i;

	while ( TRUE )
	{
		printf( "Press: 'R' to read inputs, 'U' to read changed inputs, 'C' to quit.\n\n" );
		key = getch();
		if ( key == 'C' || key == 'c' ) break;
		else if ( key == 'R' || key == 'r' )
		{
			if ( !FireFly->GetInputs( inputs ) )
			{
				printf( "\nFailure %d getting inputs.\n", (int)FireFly->GetLastError( ) );
				break;
			}

			/* Print inputs to screen. */
			printf( "Inputs   0    1    2    SW   CS\n         " );
			for ( i = 0; i < USB_IN_LENGTH; i++ ) printf( "%02X   ", inputs[i] & 0x000000FF );
			printf( "\n\n" );
		}
		else if ( key == 'U' || key == 'u' )
		{
			if ( !FireFly->GetChangedInputs( &changedInputs ) )
			{
				printf( "\nFailure %d getting inputs.\n", (int)FireFly->GetLastError( ) );
				break;
			}

			/* Print inputs to screen. */
			printf( "Changed   0    1    2    SW   CS\n" );
			printf( "          %02X   %02X   %02X   ", changedInputs.byIn[0], changedInputs.byIn[1], changedInputs.byIn[2] );
			printf( "%02X   %02X\n\n", changedInputs.bySw, changedInputs.byCs );
		}
	}
}
